require 'math'
local anim8 = require 'anim8'
require 'astar'
require 'middleclass'
local bump = require 'bump'
local sti = require 'sti'

function has_val(tabl,var)							--function used to check tables of coordinates against a coordinate
	for i, valu in ipairs(tabl) do 					--iterates over the list and returns true if it finds the variable
		if valu.x == var.x and valu.y == var.y then
			return true
		end
	end

	return false									-- returns false if it exits the loop

end

function trace(x,y,delta,map,layertag)				-- raycast algorithm using slope-intercept form

	local edgex, edgey = map.layers[layertag].width, map.layers[layertag].height  --get edge of map
	local radius = 50	-- the radius basically is how far it can "see."
	local tilesinray = {}	-- tile list

	local rise = math.sin(math.rad(delta))	--calculates the rise and run of the slope with sine and cosine of the angle
	local run = math.cos(math.rad(delta))

	local currx, curry = x, y				--defines the x and y of the ray as starting at the passed x and y variables
	local currenttile = {}
	local currtilex, currtiley

	local m = rise/run 						--creates the slope by dividing the rise and run
	local b = curry - (m * currx)			--creates y intercept
	local inc

 	if delta == 0 or delta == 180 then		-- defining the increment for x based on whether we're going backwards or forwards
 		inc = 0
	elseif delta > 180 then
		inc = -1
	else
		inc = 1
	end

	while true do 							--move the ray as fast as possible using a loop

		--local rayDist = math.floor(math.sqrt((currx^2)+(curry^2)))
		--print(rayDist)

		if currenttile.collidable == true or math.floor(currx/16) <= 0 or math.floor(curry/16) <= 0 or math.floor(currx/16) > edgex
		 or math.floor(curry/16) > edgey then --or rayDist/16 > radius or rayDist/16 > radius then
			break -- break the loop if we hit a collidable tile or the edges of the map
		end

		currenttile = map:getTileProperties(layertag,math.floor(currx/16),math.floor(curry/16))
		currtilex, currtiley = math.floor(currx/16), math.floor(curry/16) -- grab the properties of whatever tile the ray is currently in

		if delta == 180 then
			curry = curry + 1
		elseif delta == 0 then
			curry = curry - 1
		elseif delta == 90 or delta == 270 then
		else
			curry = math.floor((m * currx) + b)
		end
		currx = math.floor(currx + inc)
		if currtilex ~= math.floor(currx/16) or currtiley ~= math.floor(curry/16) then
			table.insert(tilesinray,{["x"] = math.floor(currx/16) - 1, ["y"] = math.floor(curry/16) - 1})
		end
	end

	return tilesinray
end

function love.load()

	map1 = sti.new("map1.lua",{"bump"})
	physworld = bump.newWorld()
	map1:bump_init(physworld)

	map1:addCustomLayer("Sprites",4)
	local slayer = map1.layers["Sprites"]

	local player

	for k, object in pairs(map1.objects) do
		if object.name == "player" then
			player = object
			break
		end
	end

	plsprite = love.graphics.newImage("testplyr.png")
	plsprgrid = anim8.newGrid(11,26,81,108,0,0,5)

	slayer.player = {
		anims = {},
		currentAnim = 1,
		x = player.x,
		y = player.y,
		ox = 5.5,
		oy = 22,
		visible = {}
	}

	visibleImage = love.graphics.newCanvas(math.floor(love.graphics.getWidth()/16), math.floor(love.graphics.getHeight()/16))

	--print(slayer.player.x,slayer.player.y)

	physworld:add("player",slayer.player.x,(slayer.player.y + (slayer.player.oy * 2)) - 20, 10, 10)

	table.insert(slayer.player.anims, anim8.newAnimation(plsprgrid("1-1",1),1))

	function slayer:update(dt)

		self.player.anims[1]:update(dt)

		local speed = 128

		local newx, newy = self.player.x, self.player.y

		if love.keyboard.isDown("w") then
			newy = self.player.y - speed * dt
		end

		if love.keyboard.isDown("s") then
			newy = self.player.y + speed * dt
		end

		if love.keyboard.isDown("a") then
			newx = self.player.x - speed * dt
		end

		if love.keyboard.isDown("d") then
			newx = self.player.x + speed * dt
		end
		
		local angle = 0

		if math.floor(newx/16) ~= math.floor(self.player.x/16) or math.floor(newy/16) ~= math.floor(self.player.y/16) then

			self.player.visible = {}
			local tst = 1

			repeat

				--print(self)

				for t, tile in ipairs(trace(newx, newy, angle, map1, "Tile Layer 2")) do
					table.insert(self.player.visible,tile)
				end

				angle = angle + 22.5

			until angle == 360

		end

		self.player.x, self.player.y = physworld:move("player",newx,newy)

	end

	function slayer:draw()

		self.player.anims[self.player.currentAnim]:draw(
			plsprite,
			math.floor(self.player.x),
			math.floor(self.player.y),
			0,2,2,
			self.player.ox,
			self.player.oy)
	end

	map1:removeLayer("Spawn")

end

function love.update(dt)
	map1:update(dt)
end

function love.draw()

	local screenWidth = love.graphics.getWidth()
	local screenHeight = love.graphics.getHeight()

	local player = map1.layers["Sprites"].player
	local tx = (player.x - screenWidth / 2)
	local ty = (player.y - screenHeight / 2)

	love.graphics.translate(-tx,-ty)

	map1:draw()

	love.graphics.origin()

	local dx,dy = 0,0
	local qx,qy = math.floor(tx/16),math.floor(tx/16)
	local hx, hy = math.floor((map1.layers["Sprites"].player.x + screenWidth)/16),math.floor((map1.layers["Sprites"].player.y + screenHeight)/16)

	local returnto = love.graphics.getCanvas()
	love.graphics.setCanvas(visibleImage)
	love.graphics.clear()

	repeat
		if has_val(map1.layers["Sprites"].player.visible,{['x'] = qx, ['y'] = qy}) then
			love.graphics.setColor(255,255,255,0)
			love.graphics.points(dx,dy)
		else
			love.graphics.setColor(0,0,0,128)
			love.graphics.points(dx,dy)
		end
		qy = qy + 1
		dy = dy + 1
		if qy > hy then
			qy = 1
			dy = 1
			qx = qx + 1
			dx = dx + 1
		end
	until qx == hx and qy == hy

	love.graphics.setColor(255,255,255,255)
	love.graphics.setCanvas(returnto)

	love.graphics.setBlendMode("alpha","premultiplied")
	love.graphics.draw(visibleImage,0,0,0,16,16)
	love.graphics.setBlendMode("alpha")
end